package com.ruoyi.system.utils.dbsync.runner;

import com.mysql.cj.util.StringUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.system.utils.dbsync.compare.CompareUnits;
import com.ruoyi.system.utils.dbsync.meta.Column;
import com.ruoyi.system.utils.dbsync.meta.MetaData;
import com.ruoyi.system.utils.dbsync.meta.Table;
import com.ruoyi.system.utils.dbsync.util.SqlUtil;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

public class Syncer {
   public static void sync(Properties prop) {
      String sourceHost = prop.getProperty("sourceHost");
      if (StringUtils.isEmptyOrWhitespaceOnly(sourceHost)) {
         System.out.println("sourceHost is miss");
         System.exit(-1);
      }

      String sourceUser = prop.getProperty("sourceUser");
      if (StringUtils.isEmptyOrWhitespaceOnly(sourceUser)) {
         System.out.println("sourceUser is miss");
         System.exit(-1);
      }

      String sourcePass = prop.getProperty("sourcePass");
      if (StringUtils.isEmptyOrWhitespaceOnly(sourcePass)) {
         System.out.println("sourcePass is miss");
         System.exit(-1);
      }

      String sourceSchema = prop.getProperty("sourceSchema");
      if (StringUtils.isEmptyOrWhitespaceOnly(sourceSchema)) {
         System.out.println("sourceSchema is miss");
         System.exit(-1);
      }

      String sourceCharset = prop.getProperty("sourceCharset");
      if (StringUtils.isEmptyOrWhitespaceOnly(sourceCharset)) {
         System.out.println("sourceCharset is miss");
         System.exit(-1);
      }

      String targetHost = prop.getProperty("targetHost");
      if (StringUtils.isEmptyOrWhitespaceOnly(targetHost)) {
         System.out.println("targetHost is miss");
         System.exit(-1);
      }

      String targetUser = prop.getProperty("targetUser");
      if (StringUtils.isEmptyOrWhitespaceOnly(targetUser)) {
         System.out.println("targetUser is miss");
         System.exit(-1);
      }

      String targetPass = prop.getProperty("targetPass");
      if (StringUtils.isEmptyOrWhitespaceOnly(targetPass)) {
         System.out.println("targetPass is miss");
         System.exit(-1);
      }

      String targetSchema = prop.getProperty("targetSchema");
      if (StringUtils.isEmptyOrWhitespaceOnly(targetSchema)) {
         System.out.println("targetSchema is miss");
         System.exit(-1);
      }

      String targetCharset = prop.getProperty("targetCharset");
      if (StringUtils.isEmptyOrWhitespaceOnly(targetCharset)) {
         System.out.println("targetCharset is miss");
         System.exit(-1);
      }

      String autoExecute = prop.getProperty("autoExecute");
      if (StringUtils.isEmptyOrWhitespaceOnly(autoExecute)) {
         autoExecute = "FALSE";
      }

      if (!sourceCharset.equals(targetCharset)) {
         System.out.println("source target charset not equal");
         System.out.println(-1);
      }

      DbConfig config = new DbConfig();
      config.setSourceHost(sourceHost);
      config.setSourceUser(sourceUser);
      config.setSourcePass(sourcePass);
      config.setSourceSchema(sourceSchema);
      config.setSourceCharset(sourceCharset);
      config.setTargetHost(targetHost);
      config.setTargetUser(targetUser);
      config.setTargetPass(targetPass);
      config.setTargetSchema(targetSchema);
      config.setTargetCharset(targetCharset);
      doSync(config, (String)null);
   }

   public static String doSync(String sourceHost, String sourcePass, String targetHost, String targetPass, String clientVersion) {
      DbConfig config = new DbConfig();
      config.setSourceHost(sourceHost);
      config.setSourceUser("root");
      config.setSourcePass(sourcePass);
      config.setSourceSchema("sync-client");
      config.setSourceCharset("utf8");
      config.setTargetHost(targetHost);
      config.setTargetUser("root");
      config.setTargetPass(targetPass);
      config.setTargetSchema("sync-client");
      config.setTargetCharset("utf8");
      return doSync(config, clientVersion);
   }

   public static AjaxResult executeSql(String host, String pass, String[] sql) {
      try {
         MetaData target = new MetaData();
         target.setJdbcUrl("jdbc:mysql://" + host + "/sync-client?characterEncoding=utf8");
         target.setUser("root");
         target.setPassword(pass);
         target.setSchema("sync-client");
         if (!target.connect()) {
            return AjaxResult.error("客户端数据库连接失败，请检查3306端口是否设置白名单");
         } else {
            SqlUtil.executeBatchSqlWithTransaction(target.getConn(), sql);
            return AjaxResult.success();
         }
      } catch (Exception var5) {
         Exception var4 = var5;
         Exception e = var4;
         return AjaxResult.error("执行失败 " + e.getMessage());
      }
   }

   public static String doSync(DbConfig config, String clientVersion) {
      MetaData source = new MetaData();
      source.setJdbcUrl("jdbc:mysql://" + config.getSourceHost() + "/" + config.getSourceSchema() + "?characterEncoding=" + config.getSourceCharset());
      source.setUser(config.getSourceUser());
      source.setPassword(config.getSourcePass());
      source.setSchema(config.getSourceSchema());
      if (!source.init()) {
         return "源数据库连接失败";
      } else {
         MetaData target = new MetaData();
         target.setJdbcUrl("jdbc:mysql://" + config.getTargetHost() + "/" + config.getTargetSchema() + "?characterEncoding=" + config.getTargetCharset());
         target.setUser(config.getTargetUser());
         target.setPassword(config.getTargetPass());
         target.setSchema(config.getTargetSchema());
         if (!target.init()) {
            return "目标数据库连接失败";
         } else {
            CompareUnits units = new CompareUnits(source, target);
            units.compare();
            StringBuilder sql = new StringBuilder();

            for(int i = 0; i < units.getChangeSql().size(); ++i) {
               sql.append((String)units.getChangeSql().get(i)).append("\n");
               SqlUtil.ddl(target.getConn(), (String)units.getChangeSql().get(i));
            }

            String[] tables = new String[]{"la_system_auth_menu", "la_system_auth_perm", "la_system_auth_role"};
            sql.append(copyTableData(source, target, tables));
            return sql.toString();
         }
      }
   }

   public static String copyTableData(MetaData source, MetaData target, String[] tableNames) {
      Connection sourceConnection = source.getConn();
      Connection targetConnection = target.getConn();
      PreparedStatement sourceStatement = null;
      PreparedStatement targetStatement = null;
      ResultSet resultSet = null;
      StringBuilder result = new StringBuilder();
      boolean var30 = false;

      label262: {
         try {
            var30 = true;
            targetConnection.setAutoCommit(false);
            String[] var33 = tableNames;
            int var34 = tableNames.length;

            for(int var11 = 0; var11 < var34; ++var11) {
               String tableName = var33[var11];
               List<String> columnNames = new ArrayList();
               StringBuilder insertSQL = new StringBuilder("INSERT INTO " + tableName + " (");
               StringBuilder placeholders = new StringBuilder();
               Map<String, Column> map = ((Table)target.getTables().get(tableName)).getColumns();
               map.values().forEach((column) -> {
                  String columnName = column.getName();
                  columnNames.add(columnName);
                  insertSQL.append(columnName).append(", ");
                  placeholders.append("?, ");
               });
               insertSQL.setLength(insertSQL.length() - 2);
               placeholders.setLength(placeholders.length() - 2);
               insertSQL.append(") VALUES (").append(placeholders).append(")");
               String selectSQL = "SELECT * FROM " + tableName;
               sourceStatement = sourceConnection.prepareStatement(selectSQL);
               resultSet = sourceStatement.executeQuery();
               targetStatement = targetConnection.prepareStatement(insertSQL.toString());
               String dropTableSQL = "DELETE FROM " + tableName;
               PreparedStatement delStatement = targetConnection.prepareStatement(dropTableSQL);
               delStatement.executeUpdate();

               while(resultSet.next()) {
                  for(int i = 0; i < columnNames.size(); ++i) {
                     targetStatement.setObject(i + 1, resultSet.getObject((String)columnNames.get(i)));
                  }

                  targetStatement.addBatch();
               }

               targetStatement.executeBatch();
               result.append("\n表数据复制成功: ").append(tableName);
            }

            targetConnection.commit();
            var30 = false;
            break label262;
         } catch (Exception var35) {
            Exception var31 = var35;
            Exception e = var31;

            try {
               targetConnection.rollback();
            } catch (SQLException var33) {
               SQLException var30 = var33;
               SQLException ex = var30;
               throw new RuntimeException(ex);
            }

            e.printStackTrace();
            var30 = false;
         } finally {
            if (var30) {
               try {
                  if (resultSet != null) {
                     resultSet.close();
                  }

                  if (sourceStatement != null) {
                     sourceStatement.close();
                  }

                  if (targetStatement != null) {
                     targetStatement.close();
                  }

                  if (sourceConnection != null) {
                     sourceConnection.close();
                  }

                  if (targetConnection != null) {
                     targetConnection.close();
                  }
               } catch (SQLException var31) {
                  SQLException var29 = var31;
                  SQLException e = var29;
                  e.printStackTrace();
               }

            }
         }

         try {
            if (resultSet != null) {
               resultSet.close();
            }

            if (sourceStatement != null) {
               sourceStatement.close();
            }

            if (targetStatement != null) {
               targetStatement.close();
            }

            if (sourceConnection != null) {
               sourceConnection.close();
            }

            if (targetConnection != null) {
               targetConnection.close();
               return result.toString();
            }
         } catch (SQLException var32) {
            var32.printStackTrace();
         }

         return result.toString();
      }

      try {
         if (resultSet != null) {
            resultSet.close();
         }

         if (sourceStatement != null) {
            sourceStatement.close();
         }

         if (targetStatement != null) {
            targetStatement.close();
         }

         if (sourceConnection != null) {
            sourceConnection.close();
         }

         if (targetConnection != null) {
            targetConnection.close();
         }
      } catch (SQLException var34) {
         var34.printStackTrace();
      }

      return result.toString();
   }

   public static Boolean checkConnect(String targetHost, String targetPass) {
      MetaData target = new MetaData();
      target.setJdbcUrl("jdbc:mysql://" + targetHost + "/sync-client?characterEncoding=utf8");
      target.setUser("root");
      target.setPassword(targetPass);
      target.setSchema("utf8");
      return target.connect();
   }
}
